@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------------
 # mkv.py - Matroska Streaming Video Files
 # -----------------------------------------------------------------------------
-# $Id: mkv.py 2629 2007-04-16 18:22:28Z dmeyer $
+# $Id: mkv.py 2828 2007-09-17 14:34:21Z tack $
 #
 # -----------------------------------------------------------------------------
 # kaa-Metadata - Media Metadata for Python
@@ -54,7 +54,7 @@ MATROSKA_SUBTITLES_TRACK          = 0x11
 
 MATROSKA_HEADER_ID                = 0x1A45DFA3
 MATROSKA_TRACKS_ID                = 0x1654AE6B
-MATROSKA_CUE_DATE_ID              = 0x1C53BB6B
+MATROSKA_CUES_ID                  = 0x1C53BB6B
 MATROSKA_SEGMENT_ID               = 0x18538067
 MATROSKA_SEGMENT_INFO_ID          = 0x1549A966
 MATROSKA_CLUSTER_ID               = 0x1F43B675
@@ -170,7 +170,7 @@ class EbmlEntity:
 
 
     def add_data(self, data):
-        maxlen = self.ebml_length - self.get_total_len()
+        maxlen = self.ebml_length - len(self.entity_data)
         if maxlen <= 0:
             return
         self.entity_data += data[:maxlen]
@@ -341,9 +341,9 @@ class Matroska(core.AVContainer):
                 elif ielem_id == MATROSKA_TITLE_ID:
                     self.title = ielem.get_utf8()
                 elif ielem_id == MATROSKA_DATE_UTC_ID:
-                    self.date =  unpack('!q', ielem.get_data())[0] / 10.0**9
+                    timestamp =  unpack('!q', ielem.get_data())[0] / 10.0**9
                     # Date is offset 2001-01-01 00:00:00 (timestamp 978307200.0)
-                    self.date += 978307200.0
+                    self.timestamp = int(timestamp + 978307200)
 
             self.length = duration * scalecode / 1000000000.0
 
@@ -359,7 +359,7 @@ class Matroska(core.AVContainer):
         elif elem_id == MATROSKA_SEEKHEAD_ID:
             self.process_seekhead(elem)
 
-        elif elem_id == MATROSKA_CUE_DATE_ID:
+        elif elem_id == MATROSKA_CUES_ID:
             self.has_idx = True
             
         log.debug('END: process element %s' % hex(elem_id))
