@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------------
 # riff.py - riff (avi) file parser
 # -----------------------------------------------------------------------------
-# $Id: riff.py 2581 2007-03-22 14:16:50Z tack $
+# $Id: riff.py 3213 2008-03-22 18:05:03Z tack $
 #
 # -----------------------------------------------------------------------------
 # kaa-Metadata - Media Metadata for Python
@@ -55,7 +55,6 @@ AVIINFO = {
     'IART': 'artist',
     'IPRD': 'product',
     'ISFT': 'software',
-    'ICRD': 'date',
     'ICMT': 'comment',
     'ILNG': 'language',
     'IKEY': 'keywords',
@@ -208,9 +207,10 @@ class Riff(core.AVContainer):
             ai = core.AudioStream()
             ai.samplerate = retval['nSamplesPerSec']
             ai.channels = retval['nChannels']
-            ai.samplebits = retval['nBitsPerSample']
             # FIXME: Bitrate calculation is completely wrong.
-            ai.bitrate = retval['nAvgBytesPerSec'] * 8
+            #ai.samplebits = retval['nBitsPerSample']
+            #ai.bitrate = retval['nAvgBytesPerSec'] * 8
+
             # TODO: set code if possible
             # http://www.stats.uwa.edu.au/Internal/Specs/DXALL/FileSpec/\
             #    Languages
@@ -235,7 +235,7 @@ class Riff(core.AVContainer):
             vi.width = retval['biWidth']
             vi.height = retval['biHeight']
             # FIXME: Bitrate calculation is completely wrong.
-            vi.bitrate = strh['dwRate']
+            #vi.bitrate = strh['dwRate']
             vi.fps = float(strh['dwRate']) / strh['dwScale']
             vi.length = strh['dwLength'] / vi.fps
             self.video.append(vi)
@@ -462,25 +462,23 @@ class Riff(core.AVContainer):
                 value = value.replace('\0', '').lstrip().rstrip()
                 if value:
                     retval[key] = value
-                    if key == 'IDIT':
-                        # Date the video was created
+                    if key in ('IDIT', 'ICRD'):
+                        # Timestamp the video was created
                         try:
                             # The doc says it should be a format like
                             # "Wed Jan 02 02:03:55 1990"
-                            date = time.strptime(value, "%a %b %d %Y %H:%M:%S")
+                            t = time.strptime(value, "%a %b %d %H:%M:%S %Y")
                         except ValueError:
                             try:
                                 # The Casio S500 uses "2005/12/24/ 14:11"
-                                date = time.strptime(value, "%Y/%m/%d/ %H:%M")
+                                t = time.strptime(value, "%Y/%m/%d/ %H:%M")
                             except ValueError, e:
                                 # FIXME: something different
                                 log.debug('no support for time format %s', value)
-                                date = 0
-                        if date:
-                            # format date to something similar to a date in an
-                            # EXIF header. This creates one unique way in
-                            # kaa.metadata to handle this.
-                            self.date = time.strftime("%Y:%m:%d %H:%M:%S", date)
+                                t = 0
+                        if t:
+                            # save timestamp as int
+                            self.timestamp = int(time.mktime(t))
                 i+=sz
         return retval
 
